package ControleDeType;

import java.awt.AWTError;
import java.awt.AWTException;
import java.awt.AWTPermission;
import java.awt.EventQueue;
import java.io.FileNotFoundException;
import java.io.InvalidObjectException;
import java.util.Vector;
import java.io.Serializable;

import asa.*;

public class Controle implements ParserVisitor {

	//Vecteurs recensent les variables, constantes, tableaux, methodes et les
	// erreurs
	private Vector lsVar = new Vector();

	private Vector lsTab = new Vector();

	private Vector lsCst = new Vector();

	private Vector lsMethode = new Vector();

	private Vector lsEntete = new Vector();

	private Vector vectEntetes = new Vector();

	private Vector lsErreur = new Vector();

	//Boolens indiquant au constructeur de la classe erreur s'il s'agit d'une
	// erreur de type ou de declaration
	static public final boolean undeclared = false;

	static public final boolean uncorrectType = true;

	//Entier indiquant au constructeur de la classe erreur qu'il s'agit d'un
	// entier dépassant les limites autorisées
	static public final boolean tooMuchInt = false;

	//Chaine de caractres permettant la distinction des variables locales et
	// globales
	private String actualName = new String("Globale");

	//Entier indiquant le numero de la mthode
	private int numeroMethode = 0;

	//Noeud simple referecant le noeud actuel
	private SimpleNode actualNode;

	//Objet referecant les parametres
	private Object parametres;

	static public final int intMin = -2147483648;

	static public final int intMax = 2147483647;

	/** ****************************************************************************************************** */
	/**
	 * ************************************ PARTIE I
	 * **************************************
	 */
	/** ****************************************************************************************************** */
	/** ***********************-------------------------------------------------------************************ */
	/** ********-------------------------------------------------------------------------------------********* */
	/** *---------------------------------------------------------------------------------------------------** */
	/**
	 * **------------------------------- Mthodes pour le contrle de type
	 * ---------------------------------***
	 */
	/** *---------------------------------------------------------------------------------------------------** */
	/** ********-------------------------------------------------------------------------------------********* */
	/** ***********************-------------------------------------------------------************************ */
	/** ****************************************************************************************************** */

	/** ************************************************************************************************* */
	/**
	 * ************************************** CONSTRUCTEUR
	 * *********************************************
	 */
	/** ************************************************************************************************* */

	public Controle(SimpleNode noeud) {

		//On reinitialise tous les vecteurs en supprimant les donnes qu'ils
		// contiennent.
		lsTab.removeAllElements();
		lsVar.removeAllElements();
		lsCst.removeAllElements();
		lsMethode.removeAllElements();
		lsErreur.removeAllElements();

		actualNode = noeud;
		actualNode.jjtAccept(this, parametres);

		//Afficher les erreurs suite au contrle de type
		for (int i = 0; i < lsErreur.size(); i++) {
			((erreur) lsErreur.get(i)).getErrors();
		}
	}

	public boolean noErreur() {

		boolean noErreur = false;
		if (lsErreur.isEmpty())
			noErreur = true;

		return noErreur;

	}

	public String stringErreur() {
		String erreurs = "";
		for (int i = 0; i < lsErreur.size(); i++) {
			erreurs += ((erreur) lsErreur.get(i)).getErrors() + "\n";
		}

		return erreurs;

	}

	/** ************************************************************************************************* */
	/**
	 * **************************************** TYPE TAB
	 * ***********************************************
	 */
	/** ************************************************************************************************* */

	/** ******* Chercher type tab ********* */
	public boolean findTable(String table) {

		int i = 0;
		boolean found = false;
		boolean resultat = false;

		String tableName = actualName + table;

		while (found == false && i != lsTab.size()) {
			if (((String[]) lsTab.get(i))[1].compareTo(tableName) == 0) {
				found = true;

			} else {
				i++;
			}
		}

		if (!found) {

			String globalText = new String("Globale");
			tableName = globalText + table;
			i = 0;

			while (found == false && i != lsTab.size()) {
				if (((String[]) lsTab.get(i))[1].compareTo(tableName) == 0) {
					found = true;

				} else {
					i++;
				}
			}
		}
		return found;
	}

	/** ********* Changer le nom du tab selon tab local ou global *********** */
	public String newTableName(String tab) {

		int i = 0;
		boolean found = false;
		String resultat = new String();

		String tabName = actualName + tab;

		while (found == false && i != lsTab.size()) {
			if (((String[]) lsTab.get(i))[1].compareTo(tabName) == 0) {
				found = true;
			} else {
				i++;
			}
		}

		if (found) {
			resultat = new String(tabName);
		} else {

			String globalText = new String("Globale");
			tabName = globalText + tab;
			i = 0;

			while (found == false && i != lsTab.size()) {

				if (((String[]) lsTab.get(i))[1].compareTo(tabName) == 0) {
					found = true;
				} else {
					i++;
				}
			}

			if (found) {
				resultat = new String(tabName);
			}
		}

		return resultat;

	}

	/** ************************************************************************************************* */
	/**
	 * **************************************** TYPE VAR
	 * ***********************************************
	 */
	/** ************************************************************************************************* */

	/** ************** Cherche le type var ********* */
	public boolean findVar(String var) {

		int i = 0;
		boolean found = false;

		String varName = actualName + var;

		while (found == false && i != lsVar.size()) {
			if (((String[]) lsVar.get(i))[1].compareTo(varName) == 0) {

				found = true;

			} else {

				i++;

			}
		}

		if (!found) {

			String globalText = new String("Globale");
			varName = globalText + var;
			i = 0;

			while (found == false && i != lsVar.size()) {
				if (((String[]) lsVar.get(i))[1].compareTo(varName) == 0) {

					found = true;

				} else {
					i++;
				}
			}
		}
		return found;
	}

	/**
	 * ********* Changer le nom de la var selon var locale ou globale
	 * ***********
	 */

	public String newVarName(String var) {

		int i = 0;
		boolean found = false;
		String resultat = new String();

		String varName = actualName + var;

		while (found == false && i != lsVar.size()) {
			if (((String[]) lsVar.get(i))[1].compareTo(varName) == 0) {
				found = true;
			} else {
				i++;
			}
		}

		if (found) {
			resultat = new String(varName);
		} else {

			String globalText = new String("Globale");
			varName = globalText + var;
			i = 0;

			while (found == false && i != lsVar.size()) {
				if (((String[]) lsVar.get(i))[1].compareTo(varName) == 0) {
					found = true;
				} else {
					i++;
				}
			}

			if (found) {
				resultat = new String(varName);
			}
		}

		return resultat;

	}

	/** ************************************************************************************************* */
	/**
	 * **************************************** TYPE CST
	 * ***********************************************
	 */
	/** ************************************************************************************************* */

	/** ************** Cherche le type cst ************** */
	public boolean findCst(String cst) {

		int i = 0;
		boolean found = false;

		String cstName = actualName + cst;

		while (found == false && i != lsCst.size()) {
			if (((String[]) lsCst.get(i))[1].compareTo(cstName) == 0) {
				found = true;
			} else {
				i++;
			}
		}

		if (!found) {

			String globalText = new String("Globale");
			cstName = globalText + cst;
			i = 0;

			while (found == false && i != lsCst.size()) {

				if (((String[]) lsCst.get(i))[1].compareTo(cstName) == 0) {
					found = true;
				} else {
					i++;
				}
			}

		}

		return found;
	}

	/**
	 * ********* Changer le nom de la cst selon cst locale ou globale
	 * ***********
	 */
	public String newCstName(String cst) {

		int i = 0;
		boolean found = false;
		String resultat = new String();

		String cstName = actualName + cst;

		while (found == false && i != lsCst.size()) {
			if (((String[]) lsCst.get(i))[1].compareTo(cstName) == 0) {
				found = true;
			} else {
				i++;
			}
		}

		if (found) {
			resultat = new String(cstName);
		} else {
			String globalText = new String("Globale");
			cstName = globalText + cst;
			i = 0;

			while (found == false && i != lsCst.size()) {
				if (((String[]) lsCst.get(i))[1].compareTo(cstName) == 0) {
					found = true;
				} else {
					i++;
				}
			}

			if (found) {
				resultat = new String(cstName);
			}
		}

		return resultat;
	}

	/** ************************************************************************************************* */
	/**
	 * ********************** METHODES DE RECHERCHE DANS LA TABLE DES SYMBOLES
	 * *************************
	 */
	/** ************************************************************************************************* */

	/***************************************************************************
	 * ********** Fonction qui verifie s'il s'agit d'un tableau et qui verifie
	 * s'il est bien declare dans ce cas et qui le renomme selon le parcours de
	 * l'ASA actuel (tout en verifiant s'il est declare en global ou en local).
	 * Dans le cas contraire, il effectue la mme vrification avec les table des
	 * symboles pour les variables, puis idem pour les constantes.
	 **************************************************************************/

	public boolean existe(String nomDecl, String nomPositionDecl) {
		//1er fils en string, noeud accept, noeud actuel, n du fils, ligne

		boolean resultat = false;
		String nomDeclVerification;
		//On verifie s'il s'agit d'un tableau

		System.out.println("************************");
		System.out.println("nomDecl = " + nomDecl);
		System.out.println("nomPositionDecl = " + nomPositionDecl);

		if (nomPositionDecl == "classe")
			nomDeclVerification = "Globale";
		else
			nomDeclVerification = nomPositionDecl;

		System.out.println("Tab Vide = " + lsTab.isEmpty());
		System.out.println("Var Vide = " + lsVar.isEmpty());
		System.out.println("Cst Vide = " + lsCst.isEmpty());

		if (!lsTab.isEmpty())
			for (int i = 0; i < lsTab.size(); i++)
				if (((String[]) lsTab.get(i))[1].equals(nomDeclVerification
						+ nomDecl.toString())) {

					return (true);
				}

		if (!lsVar.isEmpty())
			for (int i = 0; i < lsVar.size(); i++) {
				System.out.println("Varget = " + ((String[]) lsVar.get(i))[1]);
				System.out.println("Varget1 = " + nomDeclVerification
						+ nomDecl.toString());
				System.out.println("=== = "
						+ (((String[]) lsVar.get(i))[1]
								.equals(nomDeclVerification
										+ nomDecl.toString())));
				if (((String[]) lsVar.get(i))[1].equals(nomDeclVerification
						+ nomDecl.toString())) {

					return (true);
				}
			}

		if (!lsCst.isEmpty())
			for (int i = 0; i < lsCst.size(); i++)
				if (((String[]) lsCst.get(i))[1] == (nomDeclVerification + nomDecl
						.toString())) {
					System.out.println("Cstget = "
							+ ((String[]) lsCst.get(i))[1]);
					return (true);
				}

		return resultat;
	}

	public String verifierTypeNoeud(String nodeName, String nodeJJTAccept,
			SimpleNode node, int fils, int ligne) {
		//1er fils en string, noeud accept, noeud actuel, n du fils, ligne

		System.out.println("nodeNameNoeud = " + nodeName);
		System.out.println("nodeJJTAccept = " + nodeJJTAccept);

		String typeNoeud = null; //Type du noeud "null"
		String noeud; //Chaine de caractres representant le noeud
		String newName = new String();
		boolean cas;
		int i = 0;
		boolean found = false;

		System.out.println("OOOOOOOOOOOOOOO8 = " + node.toString());
		//On verifie s'il s'agit d'un tableau
		Node n = null;
		if (nodeName.toString().compareTo("tab") == 0)
			if (node.toString().equals(new String("var")))
				n = node.jjtGetChild(2);
			else
				n = node.jjtGetChild(1);

		if (nodeName.toString().compareTo("tab") == 0
				&& !(n instanceof ASTident)) {

			noeud = nodeJJTAccept;
			System.out.println("NOEUD FIND = " + noeud);
			cas = findTable(noeud); //retourne false ou true
			System.out.println("OOOOOOOOOOOOOOOHHHHHHHHHH = " + cas);
			//On signale tableau non declare
			if (cas == false) {

				erreur err = new erreur(undeclared, noeud + " non declare T1",
						ligne);
				System.out.println(" non declare T1");
				lsErreur.addElement(err);//ajouteNonDeclare(noeud,ligne);
			}

			if (cas == true) {

				newName = newTableName(nodeJJTAccept); //Retourne le nouveau
													   // nom selon cas local ou
													   // global
				node.jjtGetChild(fils).jjtAccept(this, newName);

				while (found == false && i != lsTab.size()) {
					if (((String[]) lsTab.get(i))[1].compareTo(newName) == 0) {
						found = true;
					} else {
						i++;
					}
				}

				//Si trouvé, on cast en une chaine de caractres le type du
				// tableau correspondant au nom

				System.out.println("FOUND = " + found);

				if (found) {
					typeNoeud = ((String[]) lsTab.get(i))[0];
				}

			}

		}

		//Sinon on verifie s'il s'agit d'une variable
		else {

			typeNoeud = nodeJJTAccept;

			if (typeNoeud == null) {

				noeud = nodeName;
				cas = findVar(noeud);
				System.out.println("CAS = " + cas);
				if (cas == true) {

					newName = newVarName(noeud); //Retourne le nouveau nom
												 // selon cas local ou global
					if (node.jjtGetChild(fils) instanceof ASTident) {

						((ASTident) node.jjtGetChild(fils)).setChaine(newName);

					}

					while (found == false && i != lsVar.size()) {
						if (((String[]) lsVar.get(i))[1].compareTo(newName) == 0) {
							found = true;
						} else {
							i++;
						}
					}

					//Si trouve, on cast en une chaine de caracteres le type du
					// tableau correspondant au nom
					if (found) {
						typeNoeud = ((String[]) lsVar.get(i))[0];
					}

				}

				//Sinon on verifie s'il s'agit d'une constante
				if (cas == false) {

					cas = findCst(noeud);

					if (cas == true) {

						newName = newCstName(noeud);
						if (node.jjtGetChild(fils) instanceof ASTident) {
							((ASTident) node.jjtGetChild(fils))
									.setChaine(newName);
						}

						typeNoeud = ((String[]) lsCst.get(i))[0];

					} else {

						erreur err = new erreur(undeclared, noeud
								+ " non declare !", ligne);
						System.out.println(noeud + " non declare !");
						lsErreur.addElement(err);
					}
				}
			}
		}
		return typeNoeud;
	}

	/***************************************************************************
	 * ********** Fonction qui verifie s'il s'agit d'un tableau et qui verifie
	 * s'il est bien declare dans ce cas et qui le renomme selon le parcours de
	 * l'ASA actuel (tout en verifiant s'il est declare en global ou en local).
	 * Dans le cas contraire, il effectue la mme vrification avec les table des
	 * symboles pour les variables, puis idem pour les constantes. S'il trouve
	 * qu'il s'agit d'une constante, une erreur est directement renvoye.
	 **************************************************************************/
	public String verifierTypeNoeudSansCst(String nodeName,
			String nodeJJTAccept, SimpleNode node, int fils, int ligne) {
		//1er fils en string, noeud accept , noeud actuel, n du fils, ligne

		String typeNoeud = null; //Type du noeud "null"
		String noeud; //Chaine de caractres representant le noeud
		String newName = new String(); //Chaine
		boolean cas;
		int i = 0;
		boolean found = false;

		System.out.println("nodeNameSsConstante = " + nodeName.toString());
		System.out.println("nodeJJTacceptSsConstante = " + nodeJJTAccept);
		System.out.println("nodeSsConstante = " + node.toString());

		//On vrifie s'il s'agit d'un tableau
		if (nodeName.toString().compareTo("tab") == 0
				&& !(node.jjtGetChild(0) instanceof ASTident)) {

			noeud = node.jjtGetChild(0).jjtGetChild(0).toString();
			cas = findTable(noeud); //retourne false ou true
			System.out.println("O = " + cas);
			//On signale tableau non declare
			if (cas == false) {

				erreur err = new erreur(undeclared, noeud
						+ " tableau non declare !", ligne);

				lsErreur.addElement(err);//ajouteNonDeclare(noeud,ligne);
			}

			if (cas == true) {

				newName = newTableName(nodeJJTAccept); //Retourne le nouveau
													   // nom selon cas local ou
													   // global
				node.jjtGetChild(fils).jjtAccept(this, newName);

				while (found == false && i != lsTab.size()) {
					if (((String[]) lsTab.get(i))[1].compareTo(newName) == 0) {
						found = true;
					} else {
						i++;
					}
				}

				//Si trouvé, on cast en une chaine de caractres le type du
				// tableau correspondant au nom

				System.out.println("FOUND = " + found);

				if (found) {
					typeNoeud = ((String[]) lsTab.get(i))[0];
				}
			}

		}

		//Sinon on vrifie s'il s'agit d'une variable
		else {

			typeNoeud = nodeJJTAccept;

			if (typeNoeud == null) {

				noeud = nodeName;
				cas = findVar(noeud);
				System.out.println("FIND NOEUD = " + noeud);
				System.out.println("CAS = " + cas);
				if (cas == true) {

					newName = newVarName(noeud); //Retourne le nom selon cas
												 // local ou global
					if (node.jjtGetChild(fils) instanceof ASTident) {

						((ASTident) node.jjtGetChild(fils)).setChaine(newName);

					}

					while (found == false && i != lsVar.size()) {
						if (((String[]) lsVar.get(i))[1].compareTo(newName) == 0) {
							found = true;
						} else {
							i++;
						}
					}

					//Si trouv, on cast en une chaine de caractres le type de
					// la variable correspondant au nom
					if (found) {
						typeNoeud = ((String[]) lsVar.get(i))[0];
					}

				}

				//Sinon on vrifie s'il s'agit d'une constante et si c'est le
				// cas, on ajoute une erreur la liste
				if (cas == false) {

					cas = findCst(noeud);

					if (found) {
						erreur err = new erreur(
								uncorrectType,
								noeud
										+ " est une constante (ne peut etre changee) !",
								ligne);
						lsErreur.addElement(err);
						typeNoeud = null;
					} else {

						erreur err = new erreur(undeclared, noeud
								+ " non declare !", ligne);
						lsErreur.addElement(err);
					}
				}
			}
		}
		return typeNoeud;
	}

	/** ****************************************************************************************************** */
	/**
	 * ************************************ PARTIE II
	 * **************************************
	 */
	/** ****************************************************************************************************** */
	/** ***********************-------------------------------------------------------************************ */
	/** ********-------------------------------------------------------------------------------------********* */
	/** *---------------------------------------------------------------------------------------------------** */
	/**
	 * *-------------------------------------------- Visiteur
	 * ---------------------------------------------**
	 */
	/** *---------------------------------------------------------------------------------------------------** */
	/** ********-------------------------------------------------------------------------------------********* */
	/** ***********************-------------------------------------------------------************************ */
	/** ****************************************************************************************************** */

	/***************************************************************************
	 * BOUCLE POUR VISITER LES NOEUDS FILS:
	 * 
	 * for (int i=0;i <node.jjtGetNumChildren();i++){
	 * node.jjtGetChild(i).jjtAccept(this,data); }
	 * 
	 * Pour tous les noeuds fils du noeud actuel (nbre de fils rcupr grce
	 * "jjtGetNumChildren()"), on les visite chacun en recuprant les donnes
	 * (grace "jjtAccept(this,data)" qui permet d'accepter le visiteur).
	 **************************************************************************/

	/** ************************************************************************************************* */
	/**
	 * ******************************************* OPERANDES
	 * *******************************************
	 */
	/** ************************************************************************************************* */

	/*-------------------------------------------------------------------------------------------*/
	/**
	 * ****************** ASTsomme equivalent ASTincrement & ASTaffectation
	 * *******************
	 */
	/*-------------------------------------------------------------------------------------------*/
	public Object visit(ASTsomme node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeudSansCst(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());
		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//Si au moins 1 des 2 noeuds est "null", on renvoie "Erreur"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			resultat = new String("Erreur");
		}

		//Sinon on compare le type du premier noeud celui du second ( l'aide de
		// la mthode de "compareTo" de la classe erreur)
		else {
			if (typeNoeud1.compareTo(typeNoeud2) == 0) {
				resultat = new String("OK"); //On renvoie "OK" si juste
			} else {
				resultat = new String("Erreur"); //Sinon on renvoie "Erreur"
				erreur err = new erreur(uncorrectType, "Type " + typeNoeud1
						+ " attendu", node.getLigne()); //Appel du constructeur
														// d'erreur, signalant
														// un type incorrect la
														// ligne dsigne et le
														// type qui est attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
			}
		}
		return resultat; //Erreur ou message retourn
	}

	public Object visit(ASTincrement node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		//On vrifie le type du noeud en vitant d'tendre la recherche aux csts
		// (impossible d'incrmenter une cst)
		typeNoeud1 = verifierTypeNoeudSansCst(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//Si le noeud est "null", on renvoie "Erreur"
		if (typeNoeud1 == null) {
			resultat = new String("Erreur");
		}

		//Sinon on compare le type du noeud au type "int"
		else {
			if (typeNoeud1.compareTo("entier") == 0) {
				resultat = new String("OK"); //On renvoie "OK" si juste
			} else {
				resultat = new String("Erreur"); //Sinon on renvoie "Erreur"
			}
		}

		return resultat; //Erreur ou message retourn
	}

	public Object visit(ASTaffectation node, Object data) {

		System.out.println("DEBUT ASTAFFECTATION");

		String typeNoeud1 = ""; //Chaine initialise "null" rcuprant le type du
								// premier noeud
		String typeNoeud2 = ""; //Chaine initialise "null" rcuprant le type du
								// second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		//On vrifie le type du premier noeud en vitant d'tendre la recherche
		// aux csts (impossible d'affecter une cst)
		typeNoeud1 = verifierTypeNoeudSansCst(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		// On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		System.out.println("typN1 = " + typeNoeud1);
		System.out.println("typN2 = " + typeNoeud2);

		//Si au moins 1 des 2 noeuds est "null", on renvoie "Erreur"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			resultat = new String("Erreur");
		}

		//Sinon on compare le type du premier noeud celui du second
		else {
			if (typeNoeud1.compareTo(typeNoeud2) == 0) {
				resultat = new String("OK"); //On renvoie "OK" si juste
			} else {
				resultat = new String("Erreur"); //Sinon on renvoie "Erreur"
				erreur err = new erreur(uncorrectType, "Type " + typeNoeud1
						+ " attendu !", node.getLigne()); //Appel du
														  // constructeur
														  // d'erreur, signalant
														  // un type incorrect
														  // la ligne dsigne et
														  // le type qui est
														  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
			}
		}

		System.out.println("FIN ASTAFFECTATION");

		return resultat; //Erreur ou message retourn
	}

	/*-----------------------------------------------------------------------------------------*/
	/**
	 * ***************** ASTsoustrac equivalent ASTplus & ASTmulti & ASTdiv
	 * ******************
	 */
	/*-----------------------------------------------------------------------------------------*/
	public Object visit(ASTplus node, Object data) {

		System.out.println("DEBUT ASTPLUS");

		String typeNoeud1; //Chaine initialise "null" rcuprant le type du
						   // premier noeud
		String typeNoeud2; //Chaine initialise "null" rcuprant le type du
						   // second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		System.out.println("Noeud1 = " + typeNoeud1);

		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		System.out.println("Noeud2 = " + typeNoeud2);

		//Si au moins 1 des 2 noeuds est "null", on signale une erreur en
		// passant "error" "true"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			error = true;
		}

		//Si les deux noeud ont un type, on vrifie si chacun est bien de type
		// "int"
		if (error == false) {
			if (typeNoeud1.compareTo("entier") != 0) {
				erreur err = new erreur(uncorrectType,
						"Type entier attendu pour l'addition ! (à gauche)",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et le type int
										  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
				error = true; //On passe "error" "true"
			}
			//Idem que pour noeud 1
			if (typeNoeud2.compareTo("entier") != 0) {
				erreur err2 = new erreur(uncorrectType,
						"Type entier attendu pour l'addition ! (à droite)",
						node.getLigne());
				lsErreur.addElement(err2);
				error = true;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "int"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("entier");
		}

		System.out.println("FIN ASTPLUS");

		return resultat; //Erreur ou "int" retourn
	}

	public Object visit(ASTsoustrac node, Object data) {

		String typeNoeud1; //Chaine initialise "null" rcuprant le type du
						   // premier noeud
		String typeNoeud2; //Chaine initialise "null" rcuprant le type du
						   // second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());
		

		//Si au moins 1 des 2 noeuds est "null", on signale une erreur en
		// passant "error" "true"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			error = true;
		}

		//Si les deux noeud ont un type, on vrifie si chacun est bien de type
		// "int"
		if (error == false) {
			if (typeNoeud1.compareTo("entier") != 0) {
				erreur err = new erreur(
						uncorrectType,
						"Type entier attendu pour la soustraction ! (à gauche)",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et le type int
										  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
				error = true; //On passe "error" "true"
			}
			//Idem que pour noeud 1
			if (typeNoeud2.compareTo("entier") != 0) {
				erreur err2 = new erreur(
						uncorrectType,
						"Type entier attendu pour la soustraction ! (à droite)",
						node.getLigne());
				lsErreur.addElement(err2);
				error = true;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "int"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("entier");
		}

		return resultat; //Erreur ou "int" retourn
	}

	public Object visit(ASTmulti node, Object data) {

		System.out.println("MULTIPLICATION");

		String typeNoeud1; //Chaine initialise "null" rcuprant le type du
						   // premier noeud
		String typeNoeud2; //Chaine initialise "null" rcuprant le type du
						   // second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud

		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		System.out.println("typeNoeud1 = " + typeNoeud1);

		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		System.out.println("typeNoeud1 = " + typeNoeud2);

		//Si au moins 1 des 2 noeuds est "null", on signale une erreur en
		// passant "error" "true"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			error = true;
		}

		//Si les deux noeud ont un type, on vrifie si chacun est bien de type
		// "int"
		if (error == false) {
			if (typeNoeud1.compareTo("entier") != 0) {
				erreur err = new erreur(
						uncorrectType,
						"Type entier attendu pour la multiplication ! (à gauche)",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et le type int
										  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
				error = true; //On passe "error" "true"
			}
			//Idem que pour noeud 1
			if (typeNoeud2.compareTo("entier") != 0) {
				erreur err2 = new erreur(
						uncorrectType,
						"Type entier attendu pour la multiplication ! (à droite)",
						node.getLigne());
				lsErreur.addElement(err2);
				error = true;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "int"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("entier");
		}

		System.out.println("FIN MULTIPLICATION");

		return resultat; //Erreur ou "int" retourn
	}

	public Object visit(ASTdiv node, Object data) {

		String typeNoeud1; //Chaine initialise "null" rcuprant le type du
						   // premier noeud
		String typeNoeud2; //Chaine initialise "null" rcuprant le type du
						   // second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//Si au moins 1 des 2 noeuds est "null", on signale une erreur en
		// passant "error" "true"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			error = true;
		}

		//Si les deux noeud ont un type, on vrifie si chacun est bien de type
		// "int"
		if (error == false) {
			if (typeNoeud1.compareTo("entier") != 0) {
				erreur err = new erreur(uncorrectType,
						"Type entier attendu pour la division ! (à gauche)",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et le type int
										  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
				error = true; //On passe "error" "true"
			}
			//Idem que pour noeud 1
			if (typeNoeud2.compareTo("entier") != 0) {
				erreur err2 = new erreur(uncorrectType,
						"Type entier attendu pour la division ! (à droite)",
						node.getLigne());
				lsErreur.addElement(err2);
				error = true;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "int"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("entier");
		}

		return resultat; //Erreur ou "int" retourn
	}

	/** ************************************************************************************************* */
	/**
	 * ********************************* OPERATEURS DE COMPARAISON
	 * *************************************
	 */
	/** ************************************************************************************************* */
	public Object visit(ASTegal node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//On vrifie le type du premier noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		//Si au moins 1 des 2 noeuds est "null", on renvoie "Erreur"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			resultat = new String("Erreur");
		}

		//Sinon on compare le type du premier noeud celui du second
		else {
			if (typeNoeud1.compareTo(typeNoeud2) == 0) {
				resultat = new String("booleen"); //On renvoie le type "boolen"
												  // si juste
			} else {
				resultat = new String("Erreur"); //Sinon on renvoie "Erreur"
				erreur err = new erreur(uncorrectType, "Comparaison "
						+ typeNoeud1 + " avec " + typeNoeud2 + " impossible !",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant une incompatibilit entre
										  // les types des noeuds la ligne
										  // dsigne.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
			}
		}

		return resultat; //Erreur retourne

	}

	public Object visit(ASTpgq node, Object data) {

		System.out.println("ASTPGQ");

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		//On vrifie le type du premier noeud

		System.out.println("ASTPGQ Noeud1 = " + node.jjtGetChild(0).toString());

		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//On vrifie le type du premier noeud

		System.out.println("ASTPGQ Noeud2 = " + node.jjtGetChild(1).toString());

		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		System.out.println("PGQNOEUD1 = " + typeNoeud1);
		System.out.println("PGQNOEUD2 = " + typeNoeud2);

		//Si au moins 1 des 2 noeuds est "null", on renvoie "Erreur"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			resultat = new String("Erreur");
		}

		//Sinon on compare le type du premier noeud celui du second
		else {
			if (typeNoeud1.compareTo(typeNoeud2) == 0) {
				resultat = new String("booleen"); //On renvoie le type "boolen"
												  // si juste
			} else {
				resultat = new String("Erreur"); //Sinon on renvoie "Erreur"
				erreur err = new erreur(uncorrectType,
						"Type entier attendu pour effectuer la comparaison !",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant le type "int" attendu
										  // pour chacun des 2 noeuds la ligne
										  // dsigne.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
			}
		}
		System.out.println("FIN_ASTPGQ");
		return resultat; //Erreur retourne
	}

	/** ************************************************************************************************* */
	/**
	 * ******************************************** NULL
	 * ***********************************************
	 */
	/** ************************************************************************************************* */
	public Object visit(ASTenil node, Object data) {
		return null;
	}

	public Object visit(ASTexnil node, Object data) {
		return null;
	}

	public Object visit(ASTinil node, Object data) {
		return null;
	}

	public Object visit(ASTomega node, Object data) {
		return null;
	}

	public Object visit(ASTrien node, Object data) {
		return "void";
	}

	public Object visit(ASTvnil node, Object data) {
		return null;
	}

	/** ************************************************************************************************* */
	/**
	 * **************************************** BOOLEENS
	 * ***********************************************
	 */
	/** ************************************************************************************************* */
	public Object visit(ASTbooleen node, Object data) {

		return new String("booleen"); //On retourne simplement la chaine
									  // "boolean"
	}

	public Object visit(ASTfaux node, Object data) {

		return new String("booleen"); //On retourne simplement la chaine
									  // "boolean"
	}

	public Object visit(ASTvrai node, Object data) {

		return new String("booleen"); //On retourne simplement la chaine
									  // "boolean"
	}

	/** ************************************************************************************************* */
	/**
	 * ********************************** OPERATEURS LOGIQUES
	 * ******************************************
	 */
	/** ************************************************************************************************* */
	public Object visit(ASTet node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		//Si au moins 1 des 2 noeuds est "null", on signale une erreur en
		// passant "error" "true"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			error = true;
		}

		//Si les deux noeud ont un type, on vrifie si chacun est bien de type
		// "boolean"
		if (error == false) {
			if (typeNoeud1.compareTo("booleen") != 0) {
				erreur err = new erreur(
						uncorrectType,
						"Type boolean attendu pour l'opérateur && ! (à gauche)",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et le type boolean
										  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
				error = true; //On passe "error" "true"
			}
			//Idem que pour noeud 2
			if (typeNoeud2.compareTo("booleen") != 0) {
				erreur err2 = new erreur(
						uncorrectType,
						"Type boolean attendu pour l'opérateur && ! (à droite)",
						node.getLigne());
				lsErreur.addElement(err2);
				error = true;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "boolean"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("booleen");
		}

		return resultat; //Erreur ou "boolean" retourn
	}

	public Object visit(ASTou node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//On vrifie le type du second noeud
		typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
				((String) node.jjtGetChild(1).jjtAccept(this, data)), node, 1,
				node.getLigne());

		//Si au moins 1 des 2 noeuds est "null", on signale une erreur en
		// passant "error" "true"
		if (typeNoeud1 == null || typeNoeud2 == null) {
			error = true;
		}

		//Si les deux noeuds ont un type, on vrifie si chacun est bien de type
		// "boolean"
		if (error == false) {
			if (typeNoeud1.compareTo("booleen") != 0) {
				erreur err = new erreur(
						uncorrectType,
						"Type boolean attendu pour l'opérateur || ! (à gauche)",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et le type boolean
										  // attendu.

				lsErreur.addElement(err); //Ajout de l'erreur au vecteur
										  // listant les erreurs.
				error = true; //On passe "error" "true"
			}
			//Idem que pour noeud 2
			if (typeNoeud2.compareTo("booleen") != 0) {
				erreur err2 = new erreur(
						uncorrectType,
						"Type boolean attendu pour l'opérateur || ! (à droite)",
						node.getLigne());
				lsErreur.addElement(err2);
				error = true;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "boolean"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("booleen");
		}

		return resultat; //Erreur ou "boolean" retourn
	}

	public Object visit(ASTnon node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//Si le noeud est "null", on signale une erreur en passant "error"
		// "true"
		if (typeNoeud1 == null) {
			error = true;
		}

		//Sinon si le noeud a un type, on vrifie si il est bien de type
		// "boolean"
		else {
			if (typeNoeud1.compareTo("booleen") != 0) {
				error = true;
			} else {
				error = false;
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type "boolean"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("booleen");
		}

		return resultat; //Erreur ou "boolean" retourn
	}

	/** ************************************************************************************************* */
	/**
	 * ******************************************** ENTIERS
	 * ********************************************
	 */
	/** ************************************************************************************************* */
	public Object visit(ASTentier node, Object data) {

		return new String("entier"); //On retourne simplement la chaine "int"
	}

	public Object visit(ASTnbre node, Object data) {

		if (data instanceof Integer) {

			System.out.println("INSTANCE INT NBRE");
			if (((Integer) data).equals(new Integer("007"))) {
				erreur err = new erreur(uncorrectType,"L'indice du tableau doit etre un entier positif !", node.getLigne());
				lsErreur.addElement(err);
			}
		}
		return new String("entier"); //On retourne simplement la chaine "int"
	}

	public Object visit(ASTmoins node, Object data) {

		String typeNoeud1 = null; //Chaine initialisée à "null" récupérant le
								  // type du noeud
		String resultat = null; //Chaine retournant l'erreur (s'il y a) après
								// contrôle
		boolean error = false; //Booléen signalant une erreur de type,
							   // initialisé à "false"
		int i;

		Integer res = new Integer("1");

		if (data instanceof Integer) {

			if (((Integer) data).equals(new Integer("006"))) {
				res = new Integer("007");
			}
		}

		//On vérifie le type du noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, res)), node, 0,
				node.getLigne());

		//Si le noeud est "null", on signale une erreur en passant "error" à
		// "true"
		if (typeNoeud1 == null) {
			error = true;
		}

		//Sinon si le noeud a un type, on vérifie s'il est bien de type "int"
		else {
			if (typeNoeud1.compareTo("entier") != 0) {

				//Dans le cas contraire, on passe error à "true, on ajoute une
				// erreur dans la liste et on renvoie le message "Erreur"
				error = true;
				erreur err = new erreur(uncorrectType,
						"Type entier attendu pour obtenir son inverse !", node
								.getLigne());
				lsErreur.addElement(err);
				resultat = new String("Erreur");
			}
		}

		//S'il ny a pas d'erreur,
		if (!error) {
			resultat = new String("entier");
		}

		return resultat; //On renvoie "Erreur" ou "int"
	}

	/** ************************************************************************************************* */
	/**
	 * ******************************** STRUCTURES CONDITIONNELLES
	 * *************************************
	 */
	/** ************************************************************************************************* */
	public Object visit(ASTsi node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du deuxime noeud
		String typeNoeud3 = null; //Chaine initialise "null" rcuprant le type
								  // du troisime noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		System.out.println("NOEUD SI = " + typeNoeud1);

		//Si le premier noeud est "null", on signale une erreur en passant
		// "error" "true"
		if (typeNoeud1 == null) {
			error = true;
			erreur err = new erreur(uncorrectType,
					"Pour la condition du si, un boolean est attendu !", node
							.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et que le type boolean
										  // est attendu.

			lsErreur.addElement(err); //Ajout de l'erreur au vecteur listant
									  // les erreurs.
		}

		//Sinon on compare le type du premier noeud au type "boolean"
		else {
			//Si le premier noeud n'est pas de type "boolean", on signale une
			// erreur en passant "error" "true"
			if (typeNoeud1.compareTo("booleen") != 0) {
				error = true;
				erreur err2 = new erreur(uncorrectType,
						"Pour la condition du si, un boolean est attendu !",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et que le type boolean
										  // est attendu.

				lsErreur.addElement(err2); //Ajout de l'erreur au vecteur
										   // listant les erreurs.
			}
		}

		// On cast en une chaine de caractres le deuxime noeud fils
		typeNoeud2 = ((String) node.jjtGetChild(1).jjtAccept(this, data));
		//Si le deuxime noeud est "null", on signale une erreur en passant
		// "error" "true"
		if (typeNoeud2 == null) {
			error = true; //Alors on retourne "error" "true"
		}
		//Sinon on compare le type du deuxime noeud au type message "OK"
		else {
			if (typeNoeud2.compareTo("OK") != 0) {
				error = true; //Alors on retourne "error" "true"
			}
		}

		// On cast en une chaine de caractres le troisime noeud fils
		typeNoeud3 = ((String) node.jjtGetChild(2).jjtAccept(this, data));
		//Si le troisime noeud est "null", on signale une erreur en passant
		// "error" "true"
		if (typeNoeud3 == null) {
			error = true; //Alors on retourne "error" "true"
		}
		//Sinon on compare le type du troisme noeud au type message "OK"
		else {
			if (typeNoeud3.compareTo("OK") != 0) {
				error = true; //Alors on retourne "error" "true"
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type message "OK"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("OK");
		}

		return resultat; //Erreur ou message "OK" retourn
	}

	public Object visit(ASTtantque node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du premier noeud
		String typeNoeud2 = null; //Chaine initialise "null" rcuprant le type
								  // du deuxime noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du premier noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//Si le premier noeud est "null", on signale une erreur en passant
		// "error" "true"
		if (typeNoeud1 == null) {
			error = true;
			erreur err = new erreur(uncorrectType,
					"Pour la condition du si, un boolean est attendu !", node
							.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et que le type boolean
										  // est attendu.

			lsErreur.addElement(err); //Ajout de l'erreur au vecteur listant
									  // les erreurs.
		}

		//Sinon on compare le type du premier noeud au type "boolean"
		else {
			//Si le premier noeud n'est pas de type "boolean", on signale une
			// erreur en passant "error" "true"
			if (typeNoeud1.compareTo("booleen") != 0) {
				error = true;
				erreur err2 = new erreur(uncorrectType,
						"Pour la condition du si, un boolean est attendu !",
						node.getLigne()); //Appel du constructeur d'erreur,
										  // signalant un type incorrect la
										  // ligne dsigne et que le type boolean
										  // est attendu.

				lsErreur.addElement(err2); //Ajout de l'erreur au vecteur
										   // listant les erreurs.
			}
		}

		// On cast en une chaine de caractres le deuxime noeud fils
		typeNoeud2 = ((String) node.jjtGetChild(1).jjtAccept(this, data));
		//Si le deuxime noeud est "null", on signale une erreur en passant
		// "error" "true"
		if (typeNoeud2 == null) {
			error = true; //Alors on retourne "error" "true"
		}
		//Sinon on compare le type du deuxime noeud au type message "OK"
		else {
			if (typeNoeud2.compareTo("OK") != 0) {
				error = true; //Alors on retourne "error" "true"
			}
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type message "OK"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("OK");
		}

		return resultat; //Erreur ou message "OK" retourn
	}

	/** ************************************************************************************************* */
	/**
	 * ************************** METHODES ET VERIFICATION DE PARAMETRES
	 * *******************************
	 */
	/** ************************************************************************************************* */

	public Object visit(ASTappelE node, Object data) {

		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean found = false; //Boolen signalant une erreur de type, initialis
							   // "false"
		int i = 0; //Variable de boucle "while"

		//On compare l'identifiant de la mthode ceux de la liste des mthodes
		// dclares
		while (i != lsMethode.size() && found == false) {
			if (((String) ((Vector) lsMethode.get(i)).get(1)).compareTo(node
					.jjtGetChild(0).toString()) == 0) {
				found = true;
			} else {
				i++;
			}

		}

		//Si rien n'est trouv, on renvoie le message indiquant que la mthode
		// appele n'existe pas (pas dclare).
		if (!found) {
			erreur err = new erreur(undeclared, "La methode"
					+ node.jjtGetChild(0).toString() + " n'existe pas !", node
					.getLigne());
			lsErreur.addElement(err); //On ajoute l'erreur la liste
			resultat = new String("Erreur");
		}

		//Sinon on va verifier que la liste des paramtres est bien la mme
		else {

			//On rcupre le noeud ASTlistexp
			String parametresActuels = ((String) node.jjtGetChild(1).jjtAccept(
					this, data));

			System.out.println("entete ======== " + parametresActuels);

			//On initialise une chaine vide (pour la cas d'une mthode sans
			// paramtres)
			String parametresDeclares = new String("");

			//Pour le nombre de paramtres de la mthode
			for (int j = 2; j < ((Vector) lsMethode.get(i)).size(); j++) {

				//Si la chaine est vide, on concatne le paramtre
				if (parametresDeclares.compareTo("") == 0) {
					parametresDeclares = parametresDeclares
							.concat((String) ((Vector) lsMethode.get(i)).get(j));
				}
				//Sinon on concatne le paramtre grce au caractre ","
				else {
					parametresDeclares = parametresDeclares.concat(",");
					parametresDeclares = parametresDeclares
							.concat((String) ((Vector) lsMethode.get(i)).get(j));
				}

			}

			System.out.println("enteteDecl ======== " + parametresDeclares);

			if (parametresDeclares.compareTo(parametresActuels) != 0) {
				erreur e = new erreur(
						uncorrectType,
						"La methode "
								+ node.jjtGetChild(0).toString()
								+ " possede un ou plusieurs parametre(s) incorrect(s) !",
						node.getLigne());
				lsErreur.addElement(e);
				resultat = new String("erreur");
			}

			//Si des arguments ont t trouvs et qu'ils ne correspondent pas ceux
			// dclars
			if (parametresDeclares.compareTo(parametresActuels) != 0) {

				//Alors on renvoie une erreur
				erreur err = new erreur(
						uncorrectType,
						"La methode "
								+ node.jjtGetChild(0).toString()
								+ " possede un ou plusieurs parametre(s) incorrect(s) !",
						node.getLigne());
				lsErreur.addElement(err);
				resultat = new String("Erreur");

			}

			//On retourne la type de la mthode
			resultat = new String(((String) ((Vector) lsMethode.get(i)).get(0)));
		}

		return resultat;
	}

	public Object visit(ASTappelI node, Object data) {

		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean found = false; //Boolen signalant une erreur de type, initialis
							   // "false"
		int i = 0; //Variable de boucle "while"

		//On compare l'identifiant de la mthode ceux de la liste des mthodes
		// dclares
		while (i != lsMethode.size() && found == false) {
			if (((String) ((Vector) lsMethode.get(i)).get(1)).compareTo(node
					.jjtGetChild(0).toString()) == 0) {
				found = true;
			} else {
				i++;
			}

		}

		//Si rien n'est trouv, on renvoie le message indiquant que la mthode
		// appele n'existe pas (pas dclare).
		if (!found) {
			erreur err = new erreur(undeclared, "La methode"
					+ node.jjtGetChild(0).toString() + " n'existe pas !", node
					.getLigne());
			lsErreur.addElement(err); //On ajoute l'erreur la liste
			resultat = new String("Erreur");
		}

		//Sinon on va vrifier que la liste des paramtres est bien la mme
		else {

			//On rcupre le noeud ASTlistexp
			String parametresActuels = ((String) node.jjtGetChild(1).jjtAccept(
					this, data));
			//On initialise une chaine vide (pour la cas d'une mthode sans
			// paramtres)
			String parametresDeclares = new String("");

			//Pour le nombre de paramtres de la mthode
			for (int j = 2; j < ((Vector) lsMethode.get(i)).size(); j++) {

				//Si la chaine est vide, on concatne le paramtre
				if (parametresDeclares.compareTo("") == 0) {
					parametresDeclares = parametresDeclares
							.concat((String) ((Vector) lsMethode.get(i)).get(j));
				}
				//Sinon on concatne le paramtre grce au caractre ","
				else {
					parametresDeclares = parametresDeclares.concat(",");
					parametresDeclares = parametresDeclares
							.concat((String) ((Vector) lsMethode.get(i)).get(j));
				}

			}

			//Si des arguments ont t trouvs et qu'ils ne correspondent pas ceux
			// dclars
			if (parametresDeclares != null && parametresActuels != null) {

				if (parametresDeclares.compareTo(parametresActuels) != 0) {

					//Alors on renvoie une erreur
					erreur err = new erreur(
							uncorrectType,
							"La methode "
									+ node.jjtGetChild(0).toString()
									+ " possede un ou plusieurs parametre(s) incorrect(s) !",
							node.getLigne());
					lsErreur.addElement(err);
					resultat = new String("Erreur");

				}
			}
		}

		//On retourne le message OK
		return new String("OK");
	}

	/** ************************************************************************************************* */
	/**
	 * **************************************** AUTRES
	 * *************************************************
	 */
	/** ************************************************************************************************* */

	/*---------------------------------------------------------------------------------------------------------*/
	/**
	 * ****************** Noeuds de ceration de tableaux, variables, constantes
	 * et mthodes *******************
	 */
	/*---------------------------------------------------------------------------------------------------------*/

	public Object visit(ASTtableau node, Object data) {

		String typeNoeud2;
		String resultat = "";

		String varTypeTab[] = new String[2];

		varTypeTab[0] = node.jjtGetChild(0).toString(); //On récupère le type
														// du tableau
		varTypeTab[1] = actualName + node.jjtGetChild(1).toString();
		//On récupère le nom actuel du tableau (selon le parcours de l'ASA)

		System.out.println("LSTAB0 = " + varTypeTab[0].toString());
		System.out.println("LSTAB1 = " + varTypeTab[1].toString());
		System.out.println("EXIST1 = " + node.jjtGetChild(1).toString());
		System.out.println("EXIST1 = " + data.toString());

		System.out.println("EXISTS = "
				+ (existe(node.jjtGetChild(1).toString(), data.toString())));

		if (!existe(node.jjtGetChild(1).toString(), data.toString())) {

			//Si le nom est bien un identifiant, on le défini en une chaine de
			// caracteres
			if ((node.jjtGetChild(1)) instanceof ASTident) {
				((ASTident) node.jjtGetChild(1)).setChaine(varTypeTab[1]);
			}

			resultat = varTypeTab[0];//On récupère le type du tableau (afin de
									 // le retourner pour le parcours de l'ASA)

			lsTab.addElement(varTypeTab); //On ajoute au vecteur le type & le
										  // nom de la variable

			if (!lsTab.isEmpty())
				for (int i = 0; i < lsTab.size(); i++) {
					System.out.println("STRING LS TAB = "
							+ (((String[]) lsTab.get(i))[1]));
				}

			node.jjtGetChild(1).jjtAccept(this, data);
			node.jjtGetChild(2).jjtAccept(this, data);

			typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(2).toString(),
					((String) node.jjtGetChild(2).jjtAccept(this,
							new Integer("006"))), node, 2, node.getLigne());

			System.out
					.println("TYPEN2 = "
							+ (typeNoeud2 == null || typeNoeud2
									.compareTo("entier") != 0));

			//Si le type est nul ou s'il n'est pas un entier
			if (typeNoeud2 == null || typeNoeud2.compareTo("entier") != 0) {
				resultat = "Erreur";
			}

		} else {
			erreur err = new erreur(undeclared, node.jjtGetChild(1).toString()
					+ " deja declare !", node.getLigne());
			lsErreur.addElement(err);
		}

		return resultat; //On retourne le message "int" ou "Erreur"
	}

	public Object visit(ASTvar node, Object data) { //0type , 1dent , 2vexp

		String typeNoeud2; //Chaine rcuprant le type du second noeud
		String varTypeName[] = new String[2]; //Tableau de taille 2
											  // enregistrant le type et le nom
											  // de la variable
		String resultat = ""; //Chaine retournant l'erreur (s'il y a) aprs
							  // contrle

		varTypeName[0] = node.jjtGetChild(0).toString(); //On rcupre le type de
														 // la variable
		varTypeName[1] = actualName + node.jjtGetChild(1).toString(); //On
																	  // rcupre
																	  // le nom
																	  // actuel
																	  // de la
																	  // variable
																	  // (selon
																	  // le
																	  // parcours
																	  // de
																	  // l'ASA)

		if (!existe(node.jjtGetChild(1).toString(), data.toString())) {

			//Si le nom est bien un identifiant, on le dfini en une chaine de
			// caracteres
			if ((node.jjtGetChild(1)) instanceof ASTident) {
				((ASTident) node.jjtGetChild(1)).setChaine(varTypeName[1]);
			}

			resultat = varTypeName[0]; //On rcupre le type de la variable (afin
									   // de le retourner pour le parcours de
									   // l'ASA)

			lsVar.addElement(varTypeName); //On ajoute au vecteur le type & le
										   // nom de la variable
			node.jjtGetChild(1).jjtAccept(this, data);

			//Si le noeud "vexp" est ASTomega, on vrifie son type

			if (node.jjtGetChild(2).toString().compareTo("omega") != 0) {

				typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(2).toString(),
						((String) node.jjtGetChild(2).jjtAccept(this, data)),
						node, 2, node.getLigne());

				//Si le type est null, on renvoie "Erreur"
				if (typeNoeud2 == null) {
					resultat = "Erreur";

				}
				//Sinon s'il est diffrent du type de l'identifiant, on
				// enregistre une erreur dans la liste
				else {
					System.out.println("vartypename = " + varTypeName[0]);
					System.out.println("type noeud = " + typeNoeud2);
					if (typeNoeud2.compareTo(varTypeName[0]) != 0) {
						resultat = "Erreur";
						erreur err = new erreur(uncorrectType,
								" Affectation impossible !", node.getLigne());
						lsErreur.addElement(err);
					}
				}
			}
			//Sinon, s'il est de type "int"
			else {

				int ident;

				if (varTypeName[0].compareTo("entier") == 0) {

					//On cast en SimpleNode l'identifiant du toisime noeud (la
					// "vexp")
					ident = ((SimpleNode) node.jjtGetChild(2)).getId();
					ASTnbre nbre = new ASTnbre(ident); //Puis on cre un nouveau
													   // noeud nombre dfini
													   // entier et 2 qu'on
													   // ajoute ce noeud (vexp)
					nbre.setInt(0);
					node.jjtAddChild(nbre, 2);
				}
				//Sinon
				else {
					//On cast en SimpleNode l'identifiant du toisime noeud (la
					// "vexp")
					ident = ((SimpleNode) node.jjtGetChild(2)).getId();
					ASTfaux faux = new ASTfaux(ident); //Puis on cre un nouveau
													   // noeud faux dfini
													   // boolen qu'on ajoute ce
													   // noeud (vexp)
					node.jjtAddChild(faux, 2);
				}
			}
		} else {
			erreur err = new erreur(undeclared, node.jjtGetChild(1).toString()
					+ " deja declare !", node.getLigne());
			lsErreur.addElement(err);
		}

		if (!lsVar.isEmpty())
			for (int i = 0; i < lsVar.size(); i++) {
				System.out.println("STRING LS VAR = "
						+ (((String[]) lsVar.get(i))[1]));
			}
		return resultat; //Erreur ou message retourn
	}

	public Object visit(ASTcst node, Object data) { //0type , 1dent , 2vexp

		String typeNoeud2; //Chaine rcuprant le type du second noeud
		String cstTypeName[] = new String[2]; //Tableau de taille 2
											  // enregistrant le type et le nom
											  // de la constante
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		cstTypeName[0] = node.jjtGetChild(0).toString(); //On rcupre le type de
														 // la constante
		cstTypeName[1] = actualName + node.jjtGetChild(1).toString(); //On
																	  // rcupre
																	  // le nom
																	  // actuel
																	  // de la
																	  // constante
																	  // (selon
																	  // le
																	  // parcours
																	  // de
																	  // l'ASA)

		//Si le nom est bien un identifiant, on le dfini en une chaine de
		// caracteres
		if ((node.jjtGetChild(1)) instanceof ASTident) {
			((ASTident) node.jjtGetChild(1)).setChaine(cstTypeName[1]);
		}

		resultat = cstTypeName[0];//On rcupre le type de la variable (afin de
								  // le retourner pour le parcours de l'ASA)

		lsCst.addElement(cstTypeName); //On ajoute au vecteur le type & le nom
									   // de la variable
		node.jjtGetChild(1).jjtAccept(this, data);

		//Si le noeud "vexp" est ASTomega, on vrifie son type
		if (node.jjtGetChild(2).toString().compareTo("omega") != 0) {

			typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(2).toString(),
					((String) node.jjtGetChild(2).jjtAccept(this, data)), node,
					2, node.getLigne());

			//Si le type est null, on renvoie "Erreur"
			if (typeNoeud2 == null) {
				resultat = "Erreur";

			}

			//Sinon s'il est diffrent du type de l'identifiant, on enregistre
			// une erreur dans la liste
			else {
				if (typeNoeud2.compareTo(cstTypeName[0]) != 0) {
					resultat = "Erreur";
					erreur err = new erreur(uncorrectType,
							" Affectation impossible !", node.getLigne());

					lsErreur.addElement(err);
				}
			}
		}

		//Sinon on retourne une erreur signalant que la constante n'est pas
		// initialise
		else {
			erreur err2 = new erreur(uncorrectType,
					" La constante n'est pas initialisee !", node.getLigne());
			lsErreur.addElement(err2);
		}

		return resultat;
	}

	public Object visit(ASTmethode node, Object data) {

		System.out.println("DEBUT METHODE");

		//On initialise un nouveau vecteur nomm "methode"
		Vector methode = new Vector();
		//On rcupre le type & le nom actuel de la mthode
		methode.addElement(node.jjtGetChild(0).toString());
		methode.addElement(node.jjtGetChild(1).toString());

		lsEntete.removeAllElements();
		vectEntetes.removeAllElements();
		//On rcupre le numro de la mthode en ajoutant "methode" la fin de
		// "lsMethode", et en rcuprant la taille du vecteur "lsMethode"
		numeroMethode = lsMethode.size();

		//Le nom actuel de la methode (selon o l'on se trouve dans le parcours
		// de l'ASA) devient l'ancien nom
		String ancienNom = actualName;
		actualName = node.jjtGetChild(1).toString(); //Et on rcupre le nom
													 // actuel

		boolean testEntete = false;
		Vector vEntetes = null;
		//BOUCLE POUR VISITER LES NOEUDS FILS
		node.jjtGetChild(1).jjtAccept(this, data);
		vEntetes = (Vector) node.jjtGetChild(2).jjtAccept(this, data);

		boolean testEnteteType = false;
		boolean testTailleEntete;
		boolean testTq = false;
		int i = 0;
		while (i < lsMethode.size() && !testEnteteType) {
			System.out.println("SI1"
					+ ((String) ((Vector) lsMethode.get(i)).get(0) == node
							.jjtGetChild(0).toString()));
			if ((String) ((Vector) lsMethode.get(i)).get(0) == node
					.jjtGetChild(0).toString()) {
				System.out.println("SI2"
						+ (((Vector) lsMethode.get(i)).get(1).toString()
								.equals(node.jjtGetChild(1).toString())));
				if (((Vector) lsMethode.get(i)).get(1).toString().equals(
						node.jjtGetChild(1).toString())) {
					System.out
							.println("SI3"
									+ (((Vector) lsMethode.get(i)).size() - 2 == lsEntete
											.size()));
					if (((Vector) lsMethode.get(i)).size() - 2 == lsEntete
							.size()) {
						int j = 2;
						testTailleEntete = true;
						testEnteteType = true;
						System.out.println("TANTQUE1"
								+ (j < ((Vector) lsMethode.get(i)).size()
										&& testTailleEntete && testEnteteType));

						System.out.println("TAILLE = "
								+ ((Vector) lsMethode.get(i)).size());
						while (j < ((Vector) lsMethode.get(i)).size()) {
							if (testEnteteType) {

								System.out.println("SI4"
										+ (((Vector) lsMethode.get(i)).get(j)
												.toString().equals(lsEntete
												.get(j - 2))));
								if (!((Vector) lsMethode.get(i)).get(j)
										.toString().equals(lsEntete.get(j - 2)))
									testEnteteType = false;
							}
							j++;

						}
					}
				}
			}
			i++;
			testTq = true;
		}

		for (int k = 0; k < lsMethode.size(); k++)
			System.out.println("LSMETHODE = " + lsMethode.elementAt(k));

		if (vEntetes != null)
			for (int k = 0; k < vEntetes.size(); k++)
				System.out.println("VENTETES = " + vEntetes.elementAt(k));

		for (int k = 0; k < lsEntete.size(); k++)
			System.out.println("LSENTETE = " + lsEntete.elementAt(k));

		System.out.println("TestEntete = " + testEnteteType);
		System.out.println("TestTq = " + testTq);

		if (lsMethode.isEmpty()) {
			lsMethode.addElement(methode); //On ajoute methode la liste des
										   // mthodes
			if (vEntetes != null)
				for (int k = 0; k < vEntetes.size(); k++)
					((Vector) lsMethode.get(numeroMethode)).addElement(vEntetes
							.elementAt(k));
		} else {
			if (testEnteteType) {
				erreur err = new erreur(tooMuchInt, " La methode "
						+ node.jjtGetChild(1) + " est deja declaree !", node
						.getLigne());
				lsErreur.addElement(err);
			} else {
				lsMethode.addElement(methode); //On ajoute methode la liste des
											   // mthodes
				if (vEntetes != null)
					for (int k = 0; k < vEntetes.size(); k++)
						((Vector) lsMethode.get(numeroMethode))
								.addElement(vEntetes.elementAt(k));

			}
		}

		node.jjtGetChild(3).jjtAccept(this, node.jjtGetChild(1).toString());
		node.jjtGetChild(4).jjtAccept(this, data);

		System.out.println("FIN METHODE");

		return null; //On retourne null
	}

	/*-----------------------------------------------------------------------------------------*/
	/**
	 * ****************** Noeuds de structure et d'organisation selon l'ASA
	 * *******************
	 */
	/*-----------------------------------------------------------------------------------------*/
	public Object visit(SimpleNode node, Object data) {
		actualNode = node;
		parametres = data;
		return null;
	}

	public Object visit(ASTanalyse node, Object data) {

		//BOUCLE POUR VISITER LES NOEUDS FILS
		//for (int i=0;i<node.jjtGetNumChildren();i++){
		node.jjtGetChild(0).jjtAccept(this, data);
		//}

		return null; //on retourne "null"

	}

	public Object visit(ASTclasse node, Object data) {

		try {
			node.jjtGetChild(0).jjtAccept(this, data);
		} catch (Exception err) {
			System.out.println(err);
		}

		node.jjtGetChild(1).jjtAccept(this, node.toString());
		node.jjtGetChild(2).jjtAccept(this, data);

		return null;
	}

	public Object visit(ASTident node, Object data) {
		return null;
	}

	public Object visit(ASTlistexp node, Object data) {

		String typeNoeud = null; //Chaine rcuprant le type du noeud
		String resultat = new String(""); //Chaine de caracteres initialise
										  // vide
		boolean found; //Boolen signalant une erreur de type

		for (int i = 0; i < node.jjtGetNumChildren(); i++) {

			//Si le noeud fils i n'est pas ASTexnil
			if (((String) node.jjtGetChild(i).toString()).compareTo("exnil") != 0) {

				//Et s'il est une liste d'expression, alors le type du noeud
				// est celui de ce fils i
				if (((String) node.jjtGetChild(i).toString())
						.compareTo("listexp") == 0) {

					typeNoeud = ((String) node.jjtGetChild(i).jjtAccept(this,
							data));

				}

				//Sinon on vrifie le type
				else {
					typeNoeud = verifierTypeNoeud(node.jjtGetChild(i)
							.toString(), ((String) node.jjtGetChild(i)
							.jjtAccept(this, data)), node, i, node.getLigne());
				}
			}

			else
				typeNoeud = "";

			//Si le type du noeud vrifi n'est pas null
			if (typeNoeud != null) {

				//S'il n'est pas de type exnil non plus, on retourne le type du
				// noeud (en le concatnant la chaine "resultat" vide)
				if (typeNoeud.compareTo("exnil") != 0) {
					if ((resultat.compareTo("") == 0)
							|| typeNoeud.compareTo("") == 0) {
						System.out.println("RESTYPE = " + typeNoeud);
						resultat = resultat.concat(typeNoeud);
						System.out.println("RES = " + resultat);
					} else {
						resultat = resultat.concat(",");
						resultat = resultat.concat(typeNoeud);
						System.out.println("RES = " + resultat);
					}
				}
			}

		}

		return resultat; //Erreur ou message retourne
	}

	public Object visit(ASTmain node, Object data) {

		actualName = "Main";
		node.jjtGetChild(0).jjtAccept(this, new String("Main"));
		node.jjtGetChild(1).jjtAccept(this, data);

		return null;
	}

	public Object visit(ASTretour node, Object data) {

		String typeNoeud1 = null; //Chaine initialise "null" rcuprant le type
								  // du noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle
		boolean error = false; //Boolen signalant une erreur de type, initialis
							   // "false"

		//On vrifie le type du noeud
		typeNoeud1 = verifierTypeNoeud(node.jjtGetChild(0).toString(),
				((String) node.jjtGetChild(0).jjtAccept(this, data)), node, 0,
				node.getLigne());

		//Si le noeud est "null" OU si le type du noeud est le message
		// "Erreur", on signale une erreur en passant "error" "true"
		if (typeNoeud1 == null || typeNoeud1.compareTo("Erreur") == 0) {
			error = true;
		}

		//Si le boolen signalant la prsence d'une erreur est "true", on renvoie
		// "Erreur", sinon on renvoie le type message "OK"
		if (error == true) {
			resultat = new String("Erreur");
		} else {
			resultat = new String("OK");
		}

		return resultat; //Erreur ou message "OK" retourn
	}

	public Object visit(ASTvars node, Object data) {

		//BOUCLE POUR VISITER LES NOEUDS FILS
		for (int i = 0; i < node.jjtGetNumChildren(); i++) {
			node.jjtGetChild(i).jjtAccept(this, data);
		}

		return null; //on retourne "null"

	}

	public Object visit(ASTdecls node, Object data) {

		//Nouvel objet de type entier initialise 0
		Integer noExec = new Integer("0");

		//BOUCLE POUR VISITER LES NOEUDS FILS:
		for (int i = 0; i < node.jjtGetNumChildren(); i++) {

			//S'il s'agit d'une mthode dclare, on ne l'excute pas et on
			// l'empile simplement.
			if (node.jjtGetChild(i).toString() == "methode") {
				node.jjtGetChild(i).jjtAccept(this, noExec);
			}
			//Sinon on continue la visite du visteur normalement
			else {
				node.jjtGetChild(i).jjtAccept(this, data);
			}
		}
		return null;
	}

	public Object visit(ASTentete node, Object data) {

		//Il s'agit ici des arguments d'une methode (si elle en a) !
		String varTypeName[] = new String[2]; //On dclare un tableau de chaines
											  // de caracteres

		//On rcupre l'identifiant
		String noeud = node.jjtGetChild(1).toString();

		// On cast le nom de la variable en le nom acuel dpendant du parcours de
		// l'ASA
		((ASTident) node.jjtGetChild(1)).setChaine(actualName + noeud);

		// On ajoute la liste de mthodes le type

		varTypeName[0] = node.jjtGetChild(0).toString(); //On indique le type
														 // de la variable
		varTypeName[1] = node.jjtGetChild(1).toString(); //On indique le nom
														 // acuel de la variable
		lsVar.addElement(varTypeName);

		lsEntete.addElement(varTypeName[0]);

		String type;

		//On continue le parcours des noeuds fils
		type = node.jjtGetChild(0).jjtAccept(this, data).toString();
		node.jjtGetChild(1).jjtAccept(this, data);

		return new String(type);
	}

	public Object visit(ASTtab node, Object data) { //0type , 1dent , 2exp

		System.out.println("DEBUT ASTTAB");

		String typeNoeud1; //Chaine recuprant le type du premier noeud
		String typeNoeud2; //Chaine recuprant le type du second noeud
		String resultat; //Chaine retournant l'erreur (s'il y a) aprs contrle

		//On accepte le visiteur pour la visite du noeud et on rcupre le nom de
		// ce noeud sous une chaine de carateres
		node.jjtGetChild(0).jjtAccept(this, data);
		resultat = new String(node.jjtGetChild(0).toString());

		//Si le paramtre est une chaine de caractres, on la cast en identifiant
		if (data instanceof String) {
			((ASTident) node.jjtGetChild(0)).setChaine(((String) data));
		}
		//Sinon on vrifie le type du second noeud
		else {
			typeNoeud2 = verifierTypeNoeud(node.jjtGetChild(1).toString(),
					((String) node.jjtGetChild(1).jjtAccept(this,
							new Integer("006"))), node, 1, node.getLigne());

			System.out.println("TABNOEUD2 = " + typeNoeud2);

			//S'il est null, on retourne un message Erreur
			if (typeNoeud2 == null) {
				resultat = "Erreur";
			}
			//Sinon s'il est de type int, on retourne aussi le message Erreur
			else {
				if (typeNoeud2.compareTo("entier") != 0) {
					resultat = "Erreur";

				}

			}
		}

		System.out.println("FIN ASTTAB");

		return resultat; //Erreur ou message retourn
	}

	public Object visit(ASTentetes node, Object data) {

		//BOUCLE POUR VISITER LES NOEUDS FILS

		vectEntetes.addElement(node.jjtGetChild(0).jjtAccept(this, data));
		node.jjtGetChild(1).jjtAccept(this, data);

		return vectEntetes; //on retourne "null"

	}

	public Object visit(ASTinstrs node, Object data) {

		//BOUCLE POUR VISITER LES NOEUDS FILS
		for (int i = 0; i < node.jjtGetNumChildren(); i++) {
			node.jjtGetChild(i).jjtAccept(this, data);
		}

		return null; //on retourne "null"

	}

}