// Source provenant du site Sun Java
package ihm;

import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.filechooser.*;

/**
* Classe filtre pour <b>JOpenDialog</b>
* @author Gervais Gwenael
*/
public class MyFileFilter extends FileFilter {

    private static String TYPE_UNKNOWN = "Type Unknown";
    private static String HIDDEN_FILE = "Hidden File";

    private Hashtable filters = null;
    private String description = null;
    private String fullDescription = null;
    private boolean useExtensionsInDescription = true;

    /**
    * Fonction de cration sans paramtres.
    */
    public MyFileFilter() {
	this.filters = new Hashtable();
    }
    /**
    * Fonction de cration avec un extension en paramtre.
    */
    public MyFileFilter(String extension) {
	this(extension,null);
    }

    /**
    * Fonction de cration avec un extension et sa description en paramtre.
    */
    public MyFileFilter(String extension, String description) {
	this();
	if(extension!=null) addExtension(extension);
 	if(description!=null) setDescription(description);
    }

    /**
    * Fonction de cration avec un tableau extensions en paramtre.
    */
    public MyFileFilter(String[] filters) {
	this(filters, null);
    }

    /**
    * Fonction de cration avec un tableau d'extension et une description en paramtre.
    */
    public MyFileFilter(String[] filters, String description) {
	this();
	for (int i = 0; i < filters.length; i++) {
	    // add filters one by one
	    addExtension(filters[i]);
	}
 	if(description!=null) setDescription(description);
    }
    
    /**
    * Fonction retournant si le fichier est accept par le filtre ou non.
    * @return booleen indiquant si le fichier est accepte par le filtre.
    */
    public boolean accept(File f) {
	if(f != null) {
	    if(f.isDirectory()) {
		return true;
	    }
	    String extension = getExtension(f);
	    if(extension != null && filters.get(getExtension(f)) != null) {
		return true;
	    };
	}
	return false;
    }

    /**
    * Fonction retournant l'extension du <i>File</i>.
    * @return l'extension du <i>File</i>
    */
     public String getExtension(File f) {
	if(f != null) {
	    String filename = f.getName();
	    int i = filename.lastIndexOf('.');
	    if(i>0 && i<filename.length()-1) {
		return filename.substring(i+1).toLowerCase();
	    };
	}
	return null;
    }

    /**
    * Fonction ajoutant une extension  accepter au filtre.
    */
    public void addExtension(String extension) {
	if(filters == null) {
	    filters = new Hashtable(5);
	}
	filters.put(extension.toLowerCase(), this);
	fullDescription = null;
    }

    /**
    * Fonction retournant la description associe au filtre.
    * @return la description du filtre.
    */
    public String getDescription() {
	if(fullDescription == null) {
	    if(description == null || isExtensionListInDescription()) {
 		fullDescription = description==null ? "(" : description + " (";
		// build the description from the extension list
		Enumeration extensions = filters.keys();
		if(extensions != null) {
		    fullDescription += "." + (String) extensions.nextElement();
		    while (extensions.hasMoreElements()) {
			fullDescription += ", ." + (String) extensions.nextElement();
		    }
		}
		fullDescription += ")";
	    } else {
		fullDescription = description;
	    }
	}
	return fullDescription;
    }

    /**
    * Fonction dfinissant la description associe au filtre.
    */
    public void setDescription(String description) {
	this.description = description;
	fullDescription = null;
    }

    /**
    * Fonction dfinissant si la list des extensions acceptes figurent dans la description associe au filtre.
    */
    public void setExtensionListInDescription(boolean b) {
	useExtensionsInDescription = b;
	fullDescription = null;
    }

    /**
    * Fonction retournant si la liste des extensions acceptes figurent dans la description associe au filtre ou non.
    * @return booleen indiquant si la liste des extension acceptees figurent dans la description associe au filtre ou non.
    */
    public boolean isExtensionListInDescription() {
	return useExtensionsInDescription;
    }
}
