package interpretation_mjj;

import asa.*;
import memoire.*;


public class interpretationMiniJaja extends Thread implements ParserVisitor{
    private Memory m = new Memory(1000);
        
    private SimpleNode rootNode;
    private String tabBreakPoints[];
    private boolean threadSuspended;
    private boolean finInterpretation;
    private int ligneEnCours;
    private String nomClasse;
    /**
     * Constructeur par dfaut, le thread est dmarr
     */
    public interpretationMiniJaja(SimpleNode n,String s){
    	rootNode=n;
        setBreakPoints(rootNode,s);
        System.out.println("s = " + s);
	threadSuspended=false;
        this.start();
        
    }
    
    /** 
     * L'interpretation est-elle termine?
     * @return true si termine et false sinon
    */
    public boolean interpretationTerminee() {
	return finInterpretation;
    }

    private void setBreakPointsRec(SimpleNode n){
        Integer num_ligne=new Integer(n.getLigne());
        for (int i=0;i<tabBreakPoints.length;i++){
            if ((num_ligne.toString().compareTo(tabBreakPoints[i]))==0){
                n.setBreakPoint();
                tabBreakPoints[i]="-1";
                
                
            }
        }
	for (int i=0;i<n.jjtGetNumChildren();i++) {
	    setBreakPointsRec((SimpleNode)n.jjtGetChild(i));
	}
    }
    
    private void setBreakPoints(SimpleNode n,String s){
        tabBreakPoints=s.split(",");
        setBreakPointsRec(n);
        
    }
    
    /**
     * Reprise de l'excution du thread
     */    
    public void resumeThread(){
	threadSuspended=false;
    }

    /**
     * Suspension de l'excution du thread
     *
     * @param node noeud courant sur lequel l'interptation est suspendue
     */      
    private void stopThread(SimpleNode node){
        //System.out.println("Suspendu sur :"+node);
        //node.removeBreakPoint();
        threadSuspended=true;
        ligneEnCours=node.getLigne();
        
        while (threadSuspended) {
		    try {
			sleep(20);
		    } catch (Exception e){
			System.out.println(e);
		    }
        }
    }
	
    /**
     * Corps d'excution du thread
     */     
    public void run(){
	finInterpretation = false;
        rootNode.jjtAccept(this,null);
	finInterpretation = true;
	//System.out.println("fini");
    }
    
    /**
     * Rcupre l'tat mmoire en cours
     *
     * @return l'tat mmoire de l'interpreteur
     */     
    public Memory getEtatMemoire(){
        return m;
    }

   /**
     * Le thread est-il suspendu ?
     *
     * @return true si le thread est suspendu, false sinon
     */  
    public boolean isThreadSuspended(){
    	
    	return threadSuspended;
    
    }
    
   /**
     * Donne la ligne sur laquelle l'interprtation est suspendue
     *
     * @return le numro de la ligne o l'interprtation est suspendue
     */  
    public int getLigneEnCours(){
    	
        return ligneEnCours;
    
    }
    
    //**************************************************************************
    //*           IMPLANTATION DE LA CLASSE ParserVisitor                      *
    //**************************************************************************
    
    public Object visit(SimpleNode noeud, Object data){
    	
    	System.out.println("data =  " + data);
    	System.out.println("NOEUD DEPART");
    	
    	return null;
    	
    }

    public Object visit(ASTanalyse noeud, Object data){ 
    	
    	System.out.println("data =  " + data);    	
    	System.out.println("ANALYSE");
    	
    	for (int i = 0 ; i < noeud.jjtGetNumChildren() ; i++)
		    
    		noeud.jjtGetChild(i).jjtAccept(this,data);
		
    	return null;
    
    }
    
    public Object visit(ASTclasse noeud, Object data) {
    	
    	System.out.println("data =  " + data);
    	System.out.println("CLASSE");
    	
    	if (noeud.isBreakPointed){
            stopThread(noeud);
        }
    	
    	nomClasse = (noeud.jjtGetChild(0).toString());
    	
    	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
		SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
		SimpleNode fils2 = (SimpleNode) noeud.jjtGetChild(2);
    	
		//fils0.jjtAccept(this, tmp);
		
    	m.declVar(fils0.toString(), noeud.toString(), "classe");
    	
    	fils1.jjtAccept(this, data);
    	
    	fils2.jjtAccept(this, data);
    	
    	
    	while (fils1.toString() == "decls")
    		fils1 = (SimpleNode) fils1.jjtGetChild(1);
    	
    
    	fils1 = (SimpleNode) fils1.jjtGetParent();
    	
    	while (fils1.toString() == "decls") {
    		if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
	    	m.retirerDecl(fils1.jjtGetChild(0).jjtGetChild(1).toString());
	    					
			fils1 = (SimpleNode) fils1.jjtGetParent();
			
    	}
    	
    	if (noeud.isBreakPointed){ 
    		
    		stopThread(noeud);  
        }
    	
			m.retirerDecl(fils0.toString());
		
		return null;
    }
    
    public Object visit(ASTtableau noeud, Object data){
	    
    	System.out.println("data =  " + data);
	    System.out.println("TABLEAU");
	    
	    
	    
	    m.declTab(noeud.jjtGetChild(1).toString(),((Integer)noeud.jjtGetChild(2).jjtAccept(this,data)).intValue(),noeud.jjtGetChild(0).toString());
		
	    
		return null;
		
    }
    
    public Object visit(ASTdecls noeud, Object data){
    
    	System.out.println("data =  " + data);
    	System.out.println("DECLS");
    	
    	
    	
    	for (int i=0;i<noeud.jjtGetNumChildren();i++) {
    		
    		noeud.jjtGetChild(i).jjtAccept(this,data);
    	
    	}
    	
		return null;
		
    }
    
    public Object visit(ASTvars noeud, Object data){       
    
    	System.out.println("data =  " + data);
    	System.out.println("VARS");
    	
    	if (noeud.isBreakPointed){
            stopThread(noeud);
        }
    	
    	for (int i = 0 ; i < noeud.jjtGetNumChildren() ; i++) {
    		
    		noeud.jjtGetChild(i).jjtAccept(this,data);
    	
    	}
		return null;
		
    }
    
    public Object visit(ASTmain noeud, Object data) {
    
    	System.out.println("data =  " + data);
    	System.out.println("MAIN");
    	
    	
    	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
		SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
		    	
    	fils0.jjtAccept(this, data);
    	
    	if (noeud.isBreakPointed){ 
    		
    		stopThread(noeud);  
        }
    	
    	fils1.jjtAccept(this, data);
			
    	
    	
    	while (fils0.toString() == "vars")
    		fils0 = (SimpleNode) fils0.jjtGetChild(1);
    	
    
    	fils0 = (SimpleNode) fils0.jjtGetParent();
    	
    	while (fils0.toString() == "vars") {
    		
    		if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
    		
				m.retirerDecl(fils0.jjtGetChild(0).jjtGetChild(1).toString());
				
				
				
			fils0 = (SimpleNode) fils0.jjtGetParent();
			
    	}
		
		return null;
    }
    
    public Object visit(ASTcst noeud, Object data){
    
    	System.out.println("data =  " + data);
    	System.out.println("CONST");
    	
    	int val_e = ((Integer) noeud.jjtGetChild(2).jjtAccept(this,data)).intValue();
    			
		m.declCst(noeud.jjtGetChild(1).toString(),new Integer(val_e).toString(),noeud.jjtGetChild(0).toString());   
		
		return null;
    }
    
    public Object visit(ASTvar noeud, Object data){
    	
    	System.out.println("data =  " + data);     	
    	System.out.println("VAR");

    	
    	
    	m.declVar(noeud.jjtGetChild(1).toString(),noeud.jjtGetChild(2).jjtAccept(this,data).toString(),noeud.jjtGetChild(0).toString());
    	
    	if (noeud.isBreakPointed){
            stopThread(noeud);
        }
    	
    	return null;
		
	}
    
    public Object visit(ASTvnil noeud, Object data) {    

    	System.out.println("data =  " + data);
    	System.out.println("VNIL");
    	
    	return null;
    	
    }
    
    public Object visit(ASTmethode noeud, Object data) {
    	
        System.out.println("data =  " + data);
        System.out.println("METHODE");
        
        if (noeud.isBreakPointed){
            stopThread(noeud);
        }
        
        m.declMeth(noeud.jjtGetChild(1).toString(),noeud,noeud.jjtGetChild(0).toString());
    	
        return noeud;
    }
    
    public Object visit(ASTinstrs noeud, Object data) { 
    
    	System.out.println("data =  " + data);
    	System.out.println("INSTRS");
    	
    	
    	
    	for (int i = 0 ; i < noeud.jjtGetNumChildren() ; i++)             
    	
    		noeud.jjtGetChild(i).jjtAccept(this,data);          
    		
    	
    	
    	return null;
    }
    
    public Object visit(ASTinil noeud, Object data) {      
    
    	System.out.println("data =  " + data);
    	System.out.println("INIL");
    	
    	return null;
    }
    
    public Object visit(ASTident node, Object data) {
    	
    	System.out.println("data =  " + data);
    	System.out.println("IDENT " + (String) m.val(node.toString()));
    	
    	return (String) m.val(node.toString());
    	
    }
        
    public Object visit(ASTentetes noeud, Object data) {
    
    	System.out.println("data =  " + data);
       	System.out.println("ENTETES");
        
       	if (noeud.isBreakPointed){
            stopThread(noeud);
        }
       	
       	for (int i = 0 ; i < noeud.jjtGetNumChildren() ; i++)
       		
       		noeud.jjtGetChild(i).jjtAccept(this,data);
       	
        	
       	return null;
    	
    }
        
        public Object visit(ASTentete noeud, Object data) {
        
        	System.out.println("data =  " + data);
        	System.out.println("ENTETE");
        	
        	if (noeud.isBreakPointed){
                stopThread(noeud);
            }
        	
        	//m.declVar(node.jjtGetChild(1).toString(),omega, node.jjtGetChild(0).toString());
        	
        	return null;
        }
        
        public Object visit(ASTtab noeud, Object data) {  
        	
        	System.out.println("data =  " + data);
        	System.out.println("TAB");
        	
        	int val_e = ((Integer) noeud.jjtGetChild(1).jjtAccept(this,data)).intValue();
        	
        	return new Integer(m.valT(noeud.jjtGetChild(0).toString(),new Integer(val_e).toString()));
        }
        
       public Object visit(ASTaffectation noeud, Object data) {
        	
        	System.out.println("data =  " + data);
        	System.out.println("AFFECTATION");
        	
        	if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
        	
    		SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
    		SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
    		
    		String val_e = fils1.jjtAccept(this,data).toString();
    		
    		System.out.println("valAffVale = " + val_e);
    		
    		if (fils0.toString() == "tab") {
    			
        		int val_e1 = ((Integer) fils0.jjtGetChild(1).jjtAccept(this,data)).intValue();
        		
        		System.out.println("valAffVale1 = " + new Integer(val_e1).toString());
        		System.out.println("fils0.jjtGetChild(0).toString()= " + fils0.jjtGetChild(0).toString());
        		
        		m.affecterValT(fils0.jjtGetChild(0).toString(), new Integer(val_e1).toString(), val_e);
    			
        	} else  
    			
    			m.affecterVal(fils0.toString(), val_e);			
    		
    		return null;
        }
        
        public Object visit(ASTincrement noeud, Object data) {
        	
        	System.out.println("data =  " + data);  
        	System.out.println("INCREMENT");
        
        	
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	
        	if ((String)(fils0.toString()) == "tab") {
    			
        		int val_e1 = ((Integer) fils0.jjtGetChild(1).jjtAccept(this,data)).intValue();
        		int val = new Integer(m.valT(fils0.toString(),new Integer(val_e1).toString())).intValue();
        		
        		m.affecterValT(fils0.jjtGetChild(0).toString(),fils0.jjtGetChild(1).jjtAccept(this,data).toString(), new Integer(val + 1).toString());
    			
        	} else {
    		
        		int inc = new Integer(m.val(fils0.toString()).toString()).intValue();
        		m.affecterVal(fils0.toString(), new Integer(inc + 1).toString());			
    		
        	}
        	
        	if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
        	
    		return null;
        }
        
        public Object visit(ASTappelI noeud, Object data) {
    		System.out.println("ASTappelI ");
    		
    		
    		if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
    			
            
    		
    		//String nomMethode = (String)node.jjtGetChild(0).jjtGetChild(0).jjtAccept(this, data).toString();
    		//System.out.println("nom mthode : " + nomMethode);
    				
    		//recherche du noeud "methode" (situ dans les dclarations de la classe
    		//Node noeudMethode = (Node)mem.val(nomMethode);
    		//Node noeudEntetes = noeudMethode.jjtGetChild(2);
    		//Node noeudEntete = noeudEntetes.jjtGetChild(0);
    		String ident = noeud.jjtGetChild(0).toString();
    		
    		System.out.println("string ident : " + ident);
    		
    		Node lexp = (Node) noeud.jjtGetChild(1);
    		
    		SimpleNode param = (SimpleNode) m.parametre(ident); 
    		
    		m.expParam(lexp, param, this);
    		
    		
    		SimpleNode decls = (SimpleNode) m.declaration(ident);
    		
    		decls.jjtAccept(this,data);
    		    		
    		SimpleNode corps = (SimpleNode) m.corps(ident);
    		
    		corps.jjtAccept(this,data);
    		
    		    				
    		// retrait des dclarations dans la mthode
    		
    		while (decls.toString() == "vars")
    			decls = (SimpleNode) decls.jjtGetChild(1);
        	
        
    		decls = (SimpleNode) decls.jjtGetParent();
        	
        	while (decls.toString() == "vars") {
        		
        			
        		
    				m.retirerDecl(decls.jjtGetChild(0).jjtGetChild(1).toString());
    			
    				
    				
    			decls = (SimpleNode) decls.jjtGetParent();
    			
        	}
    		
    		
        	while (param.toString() == "entetes")
    			param = (SimpleNode) param.jjtGetChild(1);
        	
        
    		param = (SimpleNode) param.jjtGetParent();
        	
    		System.out.println("PARAM = " + param.jjtGetChild(0).jjtGetChild(1).toString());
    		System.out.println("dans la pile : "+m.pile.get(param.jjtGetChild(0).jjtGetChild(1).toString()));
    		
        	while (param.toString() == "entetes") {
        		
        		m.retirerDecl(param.jjtGetChild(0).jjtGetChild(1).toString());
    			
    			param = (SimpleNode) param.jjtGetParent();
    			
        	}
    		
    		
    		 return null;
    	}
        
        /*public Object visit(ASTappelI noeud, Object data){  
            
        	System.out.println("data =  " + data);
        	System.out.println("APPEL_III");
        	
        	
        	String ident = noeud.jjtGetChild(0).toString(); 
        	
        	System.out.println("ident =  " + ident);
        	
        	m.expParam((ASTlistexp) noeud.jjtGetChild(1), (ASTentete) m.parametre(ident));
    		
        	noeud.jjtGetChild(0).jjtAccept(this, new Integer("0"));
        	
        	//((Node) m.declaration(ident)).jjtAccept(this, data);
    		//((Node) m.corps(ident)).jjtAccept(this, data);
    				
    		// retrait des dclarations dans la mthode
    		/*try {
				m.retirerDecl(((SimpleNode) m.declaration(ident)).jjtGetChild(0).jjtGetChild(0).jjtGetChild(1).toString());
				if (noeud.isBreakPointed){ 
	                stopThread(noeud);  
	            }
			} catch (EtatMemoireExceptionElementPasDansTable e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
    		
    		// retrait des variables de la mthode
			try {
				m.retirerDecl(((SimpleNode) m.parametre(ident)).jjtGetChild(0).jjtGetChild(0).jjtGetChild(1).toString());
				if (noeud.isBreakPointed){ 
	                stopThread(noeud);  
	            }
			} catch (EtatMemoireExceptionElementPasDansTable e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
    				
    		 return null;
        }*/
        
        public Object visit(ASTretour noeud, Object data) {
        
        	if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
        	
        	m.affecterVal((String)nomClasse, noeud.jjtGetChild(0).jjtAccept(this, data).toString());
        	
            return null;
            
        }
        
        public Object visit(ASTsomme noeud, Object data) {
        	
        	System.out.println("data =  " + data);
        	System.out.println("SOMME");
        	
        	if (noeud.isBreakPointed){ 
        		
        		stopThread(noeud);  
            }
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
    		SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
    		int valFils1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
    	        		
    		int val_e;
    	    
    		if ((String)(fils0.toString()) == "tab") {
    		
    			int val_e1 = ((Integer) fils1.jjtGetChild(1).jjtAccept(this,data)).intValue();
    			val_e = new Integer(m.valT(fils0.toString(),new Integer(val_e1).toString())).intValue();
    			
    			m.affecterValT(fils0.jjtGetChild(0).toString(),fils0.jjtGetChild(1).jjtAccept(this,data).toString(), new Integer(val_e + valFils1).toString());
    		
    		} else {
    			val_e = new Integer(m.val(fils0.toString()).toString()).intValue();
    			m.affecterVal(fils0.toString(),new Integer(val_e + valFils1).toString());
    			
    		}
    		
    		
    		return null;
    		
        }
        
        public Object visit(ASTsi noeud, Object data) {
        	
        	System.out.println("data =  " + data);
        	System.out.println("SI");
        		
    	if (((Boolean)noeud.jjtGetChild(0).jjtAccept(this,data)).booleanValue())
    	
    		noeud.jjtGetChild(1).jjtAccept(this,data);
    	
    	else
    	
    		noeud.jjtGetChild(2).jjtAccept(this,data);
    	
    	return null;
    	
        }
        
        public Object visit(ASTtantque noeud, Object data) {  
        
        	System.out.println("data =  " + data); 
        	System.out.println("TANT QUE");
        	    		
    		while (((Boolean)noeud.jjtGetChild(0).jjtAccept(this,data)).booleanValue())
    		
    			noeud.jjtGetChild(1).jjtAccept(this,data);        
    		
    		return null;
    		
        }
        
        public Object visit(ASTlistexp noeud, Object data) {    
        	
        	System.out.println("data =  " + data);
        	System.out.println("LISTEXP");
        	
        	for (int i=0;i<noeud.jjtGetNumChildren();i++) 
    		
        		noeud.jjtGetChild(i).jjtAccept(this,data);
    				
    		return null;
        }
        
        public Object visit(ASTnon noeud, Object data) {   
        	
        	System.out.println("data =  " + data);	
        	System.out.println("NON");
        	
    		SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
    			
    		boolean non0 = new Boolean(fils0.jjtAccept(this,data).toString()).booleanValue();
    			
    		return new Boolean(!non0);
    		
        }
        
        public Object visit(ASTet noeud, Object data) {
        	
        	System.out.println("data =  " + data);
        	System.out.println("ET");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
    		SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
    		    		
    		boolean et0 = new Boolean(fils0.jjtAccept(this,data).toString()).booleanValue();
    		boolean et1 = new Boolean(fils1.jjtAccept(this,data).toString()).booleanValue();
    		
    		return new Boolean(et0 && et1);
        }
        
        public Object visit(ASTou noeud, Object data) {     
        	
        	System.out.println("data =  " + data);
        	System.out.println("OU");
        	
    		SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
    		SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
    		
    		boolean ou0 = new Boolean(fils0.jjtAccept(this,data).toString()).booleanValue();
    		boolean ou1 = new Boolean(fils1.jjtAccept(this,data).toString()).booleanValue();
    		
    		return new Boolean(ou0 || ou1);
    	
        }
        
        public Object visit(ASTmoins noeud, Object data) {
        	
        	System.out.println("data =  " + data);
        	System.out.println("MOINS");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
    		
        	int moins0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	        	
        	return new Integer(- moins0);
    	
        }
        
        public Object visit(ASTegal noeud, Object data) {    
        	
        	System.out.println("data =  " + data);
        	System.out.println("EGAL");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
        	int egal0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	int egal1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
        	
        	return new Boolean(egal0 == egal1);
        }
        
        public Object visit(ASTpgq noeud, Object data) {   
        	
        	System.out.println("data =  " + data);
        	System.out.println("PGQ");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
        	int pgq0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	int pgq1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
        	
        	return new Boolean(pgq0 > pgq1);
        }
        
        public Object visit(ASTplus noeud, Object data) {     
        	
        	System.out.println("data =  " + data);
        	System.out.println("PLUS");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	
        	SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
        	int plus0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	int plus1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
        	
        	return new Integer(plus0 + plus1);
        }
        
        public Object visit(ASTsoustrac noeud, Object data) {   
        	
        	System.out.println("data =  " + data);
        	System.out.println("SOUSTRACT");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
        	int sous0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	int sous1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
        	
        	return new Integer(sous0 - sous1);
        }
        
        public Object visit(ASTmulti noeud, Object data) {   
        	
        	System.out.println("data =  " + data);
        	System.out.println("MULTI");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
        	int mul0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	int mul1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
        	
        	return new Integer(mul0 * mul1);
        }
        
        public Object visit(ASTdiv noeud, Object data) {   
        	
        	System.out.println("data =  " + data);
        	System.out.println("DIV");
        	
        	SimpleNode fils0 = (SimpleNode) noeud.jjtGetChild(0);
        	SimpleNode fils1 = (SimpleNode) noeud.jjtGetChild(1);
        	
        	int div0 = new Integer(fils0.jjtAccept(this,data).toString()).intValue();
        	int div1 = new Integer(fils1.jjtAccept(this,data).toString()).intValue();
        	
        	return new Integer(div0 / div1);
        }
        
        //mthode avec retour
        public Object visit(ASTappelE noeud, Object data) {
    		
        	System.out.println("ASTappelE : ");

        	System.out.println("PPPAAAAAUUUUSSEEE AppelI = " + noeud.getLigne());
        	System.out.println("PPPAAAAAUUUUSSEEE AppelI = " + noeud.isBreakPointed);
        	
        	
        	
        	
    		
    		ASTappelI appelI=new ASTappelI(1);
    		appelI.jjtAddChild(noeud.jjtGetChild(0),0);
    		appelI.jjtAddChild(noeud.jjtGetChild(1),1);
    		
    		appelI.jjtAccept(this,data);
    		
    		Object v = m.variableClasse(nomClasse);
    		
    		return m.variableClasse(nomClasse);
    	}
        
        public Object visit(ASTvrai noeud, Object data) {      
        	
        	System.out.println("data =  " + data);
        	System.out.println("VRAI");
        	
        	return new Boolean("true");
        }
        
        public Object visit(ASTfaux noeud, Object data) {      
        	
        	System.out.println("data =  " + data);
        	System.out.println("FALSE");
        	
        	return new Boolean("false");
        }
        
        public Object visit(ASTnbre noeud, Object data) {     
        	        	
        	System.out.println("data =  " + data);
        	System.out.println("nbre =  " + noeud.getInt());
        	System.out.println("NBRE");
        	
        	return new Integer(noeud.getInt());
        }
        
        public Object visit(ASTrien noeud, Object data) {
        	System.out.println("data =  " + data);
        	System.out.println("RIEN");
        	
        	return "void";
        }
        
        public Object visit(ASTentier noeud, Object data) {
        	System.out.println("data =  " + data);
        	System.out.println("ENTIER");
        	
        	return new String("entier");
        }
        
        public Object visit(ASTbooleen noeud, Object data) {      
        	System.out.println("data =  " + data);
        	System.out.println("BOOLEEN");
        	
        	return new String("booleen");
        }
        
        
        
        public Object visit(ASTomega noeud, Object data) {    
        	System.out.println("data =  " + data);
        	System.out.println("OMEGA");
        	
        	return "omega";
        }
        
        public Object visit(ASTenil noeud, Object data) { 
        	System.out.println("data =  " + data);
        	System.out.println("ENIL");
        	
        	return null;
        }
        
        public Object visit(ASTexnil noeud, Object data) {      
        	System.out.println("data =  " + data);
        	System.out.println("EXNIL");
        	
        	return null;
        }   
    }
